import {Directive, HostListener, Input} from '@angular/core';
import {FilterType, ICompositeFilter, IDataSource, IFilter, ISimpleFilter} from "@openharbor/data";

type FilterTypeString = `${FilterType}`;

@Directive({
  selector: '[duiSearch]',
  standalone: true
})
export class SearchDirective<TModel> {
  @Input({ required: true }) dataSource!: IDataSource<TModel>;
  @Input({ required: true }) filterPaths!: string[];
  @Input() filterType: FilterTypeString = FilterType.CONTAINS;

  filterValue?: string;
  lastUsedFilter?: IFilter;

  @HostListener('search', ['$event']) onSearch(event: Event): void {
    console.log('Search event triggered', event);
    this.filterValue = (event.target as any)?.value ?? '';
    this.applySearch();
  }

  applySearch() {
    const existingFilters = this.dataSource.filters;

    // adapt current filters to remove the previous one if exist
    // and replace with new one.
    const finalNewFilters = existingFilters
      .filter(t => t !== this.lastUsedFilter);

    if (this.filterValue) {
      const newFilter: ICompositeFilter = {
        and: true,
        type: FilterType.COMPOSITE,
        filters: this.filterPaths.map(filterPath => (<ISimpleFilter>{
          path: filterPath,
          type: this.filterType,
          value: this.filterValue,
          and: false
        }))
      };

      finalNewFilters.push(newFilter);

      // update last used filter to replace it if changed.
      this.lastUsedFilter = newFilter;
    } else {
      this.lastUsedFilter = undefined;
    }

    // execute search.
    this.dataSource.query({
      page: 1,
      filters: finalNewFilters
    });
  }
}
