import {inject, Injectable} from '@angular/core';
import {ConfirmOptions, ICommandDirectiveService, IConfirmOptions} from "@openharbor/ngx-data-ui-core";
import {Observable} from 'rxjs';
import {MatDialog, MatDialogConfig} from "@angular/material/dialog";
import {
  ConfirmDialogDefaultComponent,
  IConfirmDialogDefaultOptions
} from "../../confirm-dialog/confirm-dialog-default/confirm-dialog-default.component";

export interface IMDCommandDirectiveServiceOptions extends IConfirmOptions {
  dialogOptions?: Omit<MatDialogConfig, 'data'>;
}

@Injectable({
  providedIn: 'root'
})
export class CommandDirectiveService implements ICommandDirectiveService<IMDCommandDirectiveServiceOptions> {
  readonly dialog = inject(MatDialog);

  confirm(options: ConfirmOptions<IMDCommandDirectiveServiceOptions>): Observable<boolean> {
    const defaultOptions: Partial<IMDCommandDirectiveServiceOptions> = {
      confirmText: 'Confirm',
      cancelText: 'Cancel'
    };

    const finalOptions: IMDCommandDirectiveServiceOptions = { ...defaultOptions, ...options };

    return new Observable<boolean>(subscriber => {
      const dialogOptions : MatDialogConfig<IConfirmDialogDefaultOptions>  = {
        ...{
          autoFocus: 'dialog'
        },
        ...finalOptions.dialogOptions,
        ...{
          data: {
            title: finalOptions.title,
            message: finalOptions.message,
            confirmText: finalOptions.confirmText,
            cancelText: finalOptions.cancelText,
            success: options.success,
            loading: options.loading,
            failure: options.failure,
            formTemplate: options.formTemplate,
            form: options.form
          }
        }
      };

      const dialogRef = this.dialog.open<ConfirmDialogDefaultComponent, IConfirmDialogDefaultOptions>(ConfirmDialogDefaultComponent, dialogOptions);
      const onConfirmSub = dialogRef.componentInstance.onConfirm
        .subscribe(_ => {
          subscriber.next(true);
        });

      dialogRef.afterClosed()
        .subscribe(_ => {
          onConfirmSub.unsubscribe();
        });
    });
  }
}
